@@ -446,12 +446,14 @@ MARSS.marxss <- function(MARSS.call) {
   assign("alldefaults", alldefaults, pkg_globals)
 
   ## Check that the marssMODEL object output by MARSS.form() is ok since marxss_to_marss will go south otherwise
+  if(!identical(MARSS.call$control$trace, -1)){ # turn off all error checking if trace = -1
   if (MARSS.call$silent == 2) cat(paste("  Running is.marssMODEL on the marxss model.\n", sep = ""))
   tmp <- is.marssMODEL(marxss_object, method = MARSS.call$method)
   if (!isTRUE(tmp)) {
     cat(tmp)
     stop("Stopped in MARSS.marxss() due to problem(s) with model specification.", call. = FALSE)
   }
+  }
 
   # Put the marxss model into $model since model holds the model in the 'form' form
   MARSS.call$model <- marxss_object
@@ -753,7 +755,7 @@ MARSSinits_marxss <- function(MLEobj, inits) {
     tmp.dim <- dim(MLEobj$model$free[[elem]])[2] # how many estimated pars in marxss vers
     if (!is.null(inits[[elem]]) & !(tmp.dim == 0)) { # tmp.dim==0 means no estimated
       if (!(length(inits[[elem]]) %in% c(tmp.dim, 1))) {
-        stop(paste("Stopped in MARSSinits_marxss(): ", elem, " inits must be either a scalar (dim=NULL) or a matrix with 1 col and rows equal to the num of est values in ", elem, ".", sep = ""), call. = FALSE)
+        stop(paste("Stopped in MARSSinits_marxss(): ", elem, " inits must be either a scalar (dim=NULL) or a matrix with 1 col and rows equal to the number of estimated values in ", elem, ".", sep = ""), call. = FALSE)
       }
       if (tmp.dim != 0) inits[[elem]] <- matrix(inits[[elem]], tmp.dim, 1) else inits[[elem]] <- matrix(0, 0, 1)
     } else {
